/*
 *  Detection routines for GF1 chip and for AMD InterWave (tm) soundcard
 *  Copyright (c) by Jaroslav Kysela (Perex soft)
 */

#include "driver.h"
#include "gus.h"

#define DETECT_GUSP( port, x ) (port + ULTRA_g_u_s_##x)

static void ultra_d_gf1_write8( unsigned short port, unsigned char reg, unsigned char val )
{
  unsigned long flags;

  CLI( &flags );
  OUTB( reg, DETECT_GUSP( port, GF1REGSEL ) ); MB();
  OUTB( val, DETECT_GUSP( port, GF1DATAHIGH ) );
  STI( &flags );
}

static unsigned char ultra_d_gf1_look8( unsigned short port, unsigned char reg )
{
  unsigned long flags;
  unsigned char result;

  CLI( &flags );
  OUTB( reg, DETECT_GUSP( port, GF1REGSEL ) ); MB();
  result = INB( DETECT_GUSP( port, GF1DATAHIGH ) );
  STI( &flags );
  return result;
}

int ultra_detect_interwave( ultra_card_t *card, unsigned short port )
{
  unsigned long flags;
  unsigned char tmp, rev1, rev2;

  if ( ultra_register_ioport( card, port, 16, "InterWave (Adlib/SB compatibility)" ) < 0 )
    return -EBUSY;
  if ( ultra_register_ioport( card, port + 0x100, 12, "InterWave (synthesizer)" ) < 0 ) {
    ultra_unregister_ioports( card );
    return -EBUSY;
  }
  if ( ultra_register_ioport( card, port + 0x10c, 4, "InterWave (CS4231)" ) < 0 ) {
    ultra_unregister_ioports( card );
    return -EBUSY;
  }

#if 0
  PRINTK( "Probing InterWave at 0x%x\n", port );
#endif

  /*
   * ok.. use only INB at first stage of detection to avoid
   * trouble with wrong OUTB..
   */

  tmp = INB( DETECT_GUSP( port, MIXCNTRLREG ) );
  if ( tmp & 0x80 ) goto __nodev;
  tmp = INB( DETECT_GUSP( port, REGCNTRLS ) );
  if ( tmp & 0x04 ) goto __nodev;
  tmp = INB( DETECT_GUSP( port, GF1PAGE ) );
  if ( tmp & 0x60 ) goto __nodev;
  
  ultra_d_gf1_write8( port, ULTRA_GF1_GB_RESET, 0 );	/* reset GF1 */
  tmp = ultra_d_gf1_look8( port, ULTRA_GF1_GB_RESET );
  if ( (tmp & 0x07) != 0 ) goto __nodev;
  ultra_delay( 16 );
  ultra_d_gf1_write8( port, ULTRA_GF1_GB_RESET, 1 );	/* release reset */
  ultra_delay( 16 );
  tmp = ultra_d_gf1_look8( port, ULTRA_GF1_GB_RESET );
  if ( (tmp & 0x07) != 1 ) goto __nodev;

  CLI( &flags );
  rev1 = ultra_d_gf1_look8( port, ULTRA_GF1_GB_VERSION_NUMBER );
  ultra_d_gf1_write8( port, ULTRA_GF1_GB_VERSION_NUMBER, ~rev1 );
  rev2 = ultra_d_gf1_look8( port, ULTRA_GF1_GB_VERSION_NUMBER );
  ultra_d_gf1_write8( port, ULTRA_GF1_GB_VERSION_NUMBER, rev1 );
  STI( &flags );

  if ( ( rev1 & 0xf0 ) == ( rev2 & 0xf0 ) &&
       ( rev1 & 0x0f ) != ( rev2 & 0x0f ) ) {
    return 0;		/* ok.. We have InterWave board */
  }
  __nodev:
  ultra_unregister_ioports( card );
  return -ENODEV;
}
